<?php

namespace Ep\Gath\Plugin;

/**
 * Prepoznaje slike iz Gatha kao attachmente.
 *
 * @package Ep\Gath\Plugin
 * @author Elektronika Perhoč <web@ep.hr>
 * @link http://gitlab.ep.local/ep-framework/gath
 */
class Images extends AbstractGathPlugin
{
    /**
     * Dodaje akcije i filtere u WordPress.
     *
     * @return void
     */
    public function __invoke(): void
    {
        add_filter('wp_get_attachment_image_src', [$this, 'get_attachment_image_src'], 10, 3);
    }

    /**
     * Ako je kao $attachment_id poslan string koji počinje sa 'gath-', vraća
     * URL te slike na servisu sa slikama.
     *
     * @param $image
     * @param $attachment_id
     * @param $size
     *
     * @return array
     */
    public function get_attachment_image_src($image, $attachment_id, $size)
    {
        if (preg_match('/^gath-(.*)$/', $attachment_id, $matches)) {
            if (strlen($matches[1]) > 0) {
                $imageUrl = $this->getImageUrl($matches[1], $size);
                return [$imageUrl, null, null];
            }
        } elseif (preg_match('/^external-(.*)$/', $attachment_id, $matches)) {
            if (strlen($matches[1]) > 0) {
                $imageUrl = $this->getExternalImageUrl($matches[1]);
                return [$imageUrl, null, null];
            }
        }

        return $image;
    }

    /**
     * Vraca string https://server.tld/external/external-host.tld/path/slika.jpg
     *
     * @param $url
     * @return string
     */
    public function getExternalImageUrl($url)
    {
        return sprintf('https://%s/external/%s', $_SERVER['HTTP_HOST'], $url);
    }

    /**
     * Generira URL za datu sliku.
     *
     * @param $path
     * @param string $size
     * @param bool $cropped
     *
     * @return string
     */
    public function getImageUrl($path, $size = 'full', $cropped = false)
    {
        $baseUri = rtrim(preg_replace('/^https?:/', '', $this->gath->getImageBaseUri()), '/');

        if (empty($baseUri)) {
            throw new \RuntimeException('Nije zadan osnovni URI servisa sa slikama.');
        }

        $parts = [];

        if ($size !== 'full') {
            if (is_string($size)) {
                global $_wp_additional_image_sizes;

                if (!isset($_wp_additional_image_sizes[$size]) && !in_array($size, ['thumbnail', 'medium', 'medium_large', 'large'])) {
                    throw new \RuntimeException(sprintf("Nepoznata veličina slike '%s'.", $size));
                }

                if (in_array($size, ['thumbnail', 'medium', 'medium_large', 'large'])) {
                    $width = get_option("{$size}_size_w");
                    $height = get_option("{$size}_size_h");
                    $cropped = (bool)get_option("{$size}_crop");
                } else {
                    $width = $_wp_additional_image_sizes[$size]['width'];
                    $height = $_wp_additional_image_sizes[$size]['height'];
                    $cropped = $_wp_additional_image_sizes[$size]['crop'];
                }
            } else if (is_array($size)) {
                [$width, $height] = $size;
            } else {
                throw new \RuntimeException(sprintf(
                    "Neispravan parametar \$size, mora biti string ili array a dobio sam %s.",
                    is_object($size) ? get_class($size) : gettype($size)
                ));
            }

            $parts[] = sprintf('%sx%s', $width, $height);
        }

        if ($cropped && preg_match('/carparts/', $baseUri)) {
            array_unshift($parts, 'thumb');
        } else if (preg_match('/wptest/', $baseUri)) {
            array_unshift($parts, 'images');
        }

        $parts[] = implode('/', array_map('rawurlencode', explode('/', $path)));

        return sprintf('%s/%s', $baseUri, implode('/', $parts));
    }
}
